function [X,r,S,DX,DY]=mdsi(Dmin,Dmax,p,W,X0,r0);
% Metric Multidimensional Scaling of Interval Data
% [X,r]=mdsi(Dmin,Dmax,p,W,X0,r0) 
% gives a representation of dissimilarity measurements among objects 
% in a p-dimensional space. The dissimilarities are assumed to lie 
% within an interval defined by by the matrices Dmin and Dmax. 
% The objects are represented as hypersperes defined by a center 
% and a radius. Since the approach is metric, a linear 
% relationship between input dissimilarities and reconstructed 
% distances is assumed.
%
%  ***** INPUTS
%    Dmin,Dmax: min and max dissimilarity matrices, dimension (m,m)
%	 p: dimension of the configuration
%	 W: optional (default = 1-eye(n,n)) 
%	     (allows incomplete input matrices to be used). 
%	     W(i,j) = 0 if d(i,j) available
%		        = 1 otherwise
%	     W(i,i) = 0 for all i
%	X0,r0 (optional) : initial configuration (centers and radii)
%
%   **** OUTPUTS
%	X: centers of the hyperspheres (m,p)
%	r: radii of the hyperspheres (m,1)
%   S: stress
%   DX: vector of min and max dissimilarities
%   DY: vector of min and max reconstructed distances, same dimension as DX
%   (for Sheppard diagram)
%
%
% REQUIRES: harris.m, stress.m, tabdist.m
%
% Reference: T.  Denoeux, M. Masson, Multidimensional scaling of interval-valed dissimilarity data, 
% Pattern Recognition Letters, 21, 3-92, 2000. 

% INITIALISATION

m = max(size(Dmin));
if nargin==3;
    W=1-eye(m,m);
end;

if nargin<=4
    X = randn(m,p) ;
    r = 0.01*ones(m,1);

    % NORMALISATION

    moyX=mean(X);
    matmoy=(moyX'*ones(1,m))';
    X=X-matmoy;
    coef=sqrt(sum(diag(X*X'))/m);
    X=X/coef;
else
    X=X0;
    r=r0;
end;

% OPTIMISATION

param=[reshape(X,m*p,1);r];

options(1) = 1;
options(2) = 10000;
options(3) = 1e-6;
options(4) = 10;
[P,S] = harris('stress',param,options,Dmin,Dmax,W,m,p);


X=P(1:m*p);
X = reshape(X,m,p);
r = P(m*p+1:m*(p+1));
r=r.^2;

D = tabdist(X);
R = ones(m,1)*r';
D2 = D+R+R';
D1 = max(0,D-R-R');
I=find(W);
DY=[D1(I);D2(I)];
DX=[Dmin(I);Dmax(I)];

